/*
 * MIT License
 *
 * Copyright (c) 2017 Meilhaus Electronic GmbH
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <linux/serial.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>

#define TIOCGMPIO										0x5490
#define TIOCSMPIO										0x5491

#define MPIO0											0x01
#define MPIO1											0x02
#define MPIO2											0x04
#define MPIO3											0x08
#define MPIO4											0x10
#define MPIO5											0x20
#define MPIO6											0x40
#define MPIO7											0x80

#define MPIOVAL											0x100
#define MPIOINV											0x200
#define MPIODIR											0x400

int getMPIO(int fd, int mode)
{
	// Get current status
	return ioctl(fd, TIOCGMPIO, &mode);
}

int setMPIO(int fd, int mode, int value)
{
	mode |= value;
	// Set current status
	return ioctl(fd, TIOCSMPIO, &mode);
}

int main() {
	int status;
	int fd;
	char *serialdev = "/dev/ttyME0";

	if ((fd = open(serialdev, O_RDWR)) == -1) {
		perror("Open port");
		return 1;
	}

	status = getMPIO(fd, MPIOVAL);
	if (status < 0) {
		perror("Get MPIO value");
		goto error;
	}

	printf("MPIO value: ");
	printf("0x%x\n", status);

	status = getMPIO(fd, MPIOINV);
	if (status < 0) {
		perror("Get MPIO inversion");
		goto error;
	}

	printf("MPIO inversion: ");
	printf("0x%x\n", status);

	status = getMPIO(fd, MPIODIR);
	if (status < 0) {
		perror("Get MPIO direction");
		goto error;
	}

	printf("MPIO direction: ");
	printf("0x%x\n", status);

	//Setting state of outputs
	status = setMPIO(fd, MPIOVAL, MPIO3 | MPIO7);
	if (status < 0) {
		perror("Set MPIO value");
		goto error;
	}

	//Set input inversion status
	//status = setMPIO(fd, MPIOINV, );
	//if (status < 0) {
	//	perror("Set MPIO inversion");
	//	goto error;
	//}

	//Setting MPIO bit high means it will be setup as input. A low bit means MPIO is output.
	status = setMPIO(fd, MPIODIR, MPIO2 | MPIO6);
	if (status < 0) {
		perror("Set MPIO direction");
		goto error;
	}

	sleep(1); // pause 1 second, wait for settings to get effective

	status = getMPIO(fd, MPIOVAL);
	if (status < 0) {
		perror("Get MPIO value");
		goto error;
	}

	printf("MPIO value: ");
	printf("0x%x\n", status);

error:
	if (close(fd) < 0)
		perror("Close port");
	return 0;
}
